@@ -156,9 +156,19 @@ struct partList {		/*used by the backup system */
 #define	VOK 0x02
 
 /* Values for the UV_RestoreVolume flags parameter */
-#define RV_FULLRST 0x1
-#define RV_OFFLINE 0x2
-#define RV_RDONLY  0x10000
+/* Also used for UV_CopyVolume and UV_CloneVolume */
+#define RV_FULLRST	0x00001
+#define RV_OFFLINE	0x00002
+#define RV_CRDUMP	0x00010
+#define RV_CRKEEP	0x00020
+#define RV_CRNEW	0x00040
+#define RV_LUDUMP	0x00100
+#define RV_LUKEEP	0x00200
+#define RV_LUNEW	0x00400
+#define RV_RDONLY	0x10000
+#define RV_CPINCR	0x20000
+#define RV_NOVLDB	0x40000
+#define RV_NOCLONE	0x80000
 
 extern afs_uint32 vsu_GetVolumeID(char *astring, struct ubik_client *acstruct, afs_int32 *errp);
 extern int vsu_ExtractName(char rname[], char name[]);
